@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000901
+#define IP_VS_VERSION_CODE            0x000902
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -35,13 +35,24 @@
  */
 #define IP_VS_TEMPLATE_TIMEOUT  6*60*HZ
 
+/*
+ *      IPVS sync daemon states
+ */
+#define IP_VS_STATE_NONE    0           /* daemon is stopped */
+#define IP_VS_STATE_MASTER  1           /* started as master */
+#define IP_VS_STATE_BACKUP  2           /* started as backup */
+
 #define IP_VS_SCHEDNAME_MAXLEN         16
+#define IP_VS_IFNAME_MAXLEN            16
 
 struct ip_vs_rule_user {
 	/* global options */
 	int             tcp_timeout;    /* timeout values */
 	int             tcp_fin_timeout;
 	int             udp_timeout;
+	int             state;          /* sync daemon state */
+	char            mcast_ifn[IP_VS_IFNAME_MAXLEN];
+					/* multicast interface name */
 
 	/* virtual service options */
 	u_int16_t	protocol;
@@ -77,7 +88,9 @@ struct ip_vs_rule_user {
 #define IP_VS_SO_SET_DELDEST	(IP_VS_BASE_CTL+8)
 #define IP_VS_SO_SET_EDITDEST	(IP_VS_BASE_CTL+9)
 #define IP_VS_SO_SET_TIMEOUTS	(IP_VS_BASE_CTL+10)
-#define IP_VS_SO_SET_MAX	IP_VS_SO_SET_TIMEOUTS
+#define IP_VS_SO_SET_STARTDAEMON (IP_VS_BASE_CTL+11)
+#define IP_VS_SO_SET_STOPDAEMON  (IP_VS_BASE_CTL+12)
+#define IP_VS_SO_SET_MAX	IP_VS_SO_SET_STOPDAEMON
 
 #define IP_VS_SO_GET_VERSION	IP_VS_BASE_CTL
 #define IP_VS_SO_GET_MAX	IP_VS_SO_GET_VERSION
@@ -86,16 +99,18 @@ struct ip_vs_rule_user {
 /*
  *      IPVS Connection Flags
  */
-#define IP_VS_CONN_F_FWD_MASK         0x0003    /* mask for the fwd methods */
+#define IP_VS_CONN_F_FWD_MASK         0x0007    /* mask for the fwd methods */
 #define IP_VS_CONN_F_MASQ	      0x0000    /* masquerading */
 #define IP_VS_CONN_F_LOCALNODE	      0x0001    /* local node */
 #define IP_VS_CONN_F_TUNNEL	      0x0002    /* tunneling */
 #define IP_VS_CONN_F_DROUTE           0x0003    /* direct routing */
+#define IP_VS_CONN_F_BYPASS           0x0004    /* cache bypass */
 #define IP_VS_CONN_F_HASHED	      0x0040	/* hashed entry */
 #define IP_VS_CONN_F_NOOUTPUT         0x0080    /* no output packets */
 #define IP_VS_CONN_F_INACTIVE         0x0100    /* not established */
 #define IP_VS_CONN_F_OUT_SEQ          0x0200    /* must do output seq adjust */
 #define IP_VS_CONN_F_IN_SEQ           0x0400    /* must do input seq adjust */
+#define IP_VS_CONN_F_SEQ_MASK         0x0600    /* in/out sequence mask */
 #define IP_VS_CONN_F_NO_CPORT         0x0800    /* no client port set yet */
 
 #define NFMARK_IPVS_PROPERTY	      -0x100
@@ -110,6 +125,7 @@ struct ip_vs_rule_user {
 #include <asm/atomic.h>                 /* for struct atomic_t */
 #include <linux/netdevice.h>		/* for struct neighbour; */
 #include <net/dst.h>			/* for struct dst_entry */
+#include <net/route.h>			/* for ip_route_output */
 
 
 #ifdef CONFIG_IP_VS_DEBUG
@@ -123,6 +139,7 @@ extern int ip_vs_get_debug_level(void);
 #define IP_VS_DBG(level, msg...)  do {} while (0)
 #endif
 
+#define IP_VS_BUG() BUG()
 #define IP_VS_ERR(msg...) printk(KERN_ERR "IPVS: " ## msg )
 #define IP_VS_INFO(msg...) printk(KERN_INFO "IPVS: " ## msg )
 #define IP_VS_WARNING(msg...) \
@@ -183,6 +200,8 @@ enum {
 	NET_IPV4_VS_USE_DST=21,
 	NET_IPV4_VS_CACHE_BYPASS=22,
 	NET_IPV4_VS_EXPIRE_NODEST_CONN=23,
+	NET_IPV4_VS_SYNC_THRESHOLD=24,
+	NET_IPV4_VS_LAST
 };
 
 
@@ -330,11 +349,11 @@ struct ip_vs_dest {
 	__u32			addr;	  /* IP address of real server */
 	__u16			port;	  /* port number of the service */
 	unsigned		flags;	  /* dest status flags */
-	atomic_t		weight;	/* server weight */
+	atomic_t		weight;	  /* server weight */
 	atomic_t		conn_flags;	/* flags to copy to conn */
 	atomic_t		activeconns;	/* active connections */
-	atomic_t		inactconns;	/* inactive connections */
-	atomic_t		refcnt;	/* reference counter */
+	atomic_t		inactconns;     /* inactive connections */
+	atomic_t		refcnt;	        /* reference counter */
 	struct ip_vs_stats      stats;          /* statistics */
 
 	/* for destination cache */
@@ -453,9 +472,6 @@ extern int ip_vs_set_state(struct ip_vs_conn *cp, int state_off,
 extern int ip_vs_conn_listen(struct ip_vs_conn *cp);
 extern int ip_vs_check_template(struct ip_vs_conn *ct);
 extern void ip_vs_secure_tcp_set(int on);
-extern void ip_vs_bind_bypass_xmit(struct ip_vs_conn *cp);
-extern void ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest);
-extern void ip_vs_unbind_dest(struct ip_vs_conn *cp);
 extern void ip_vs_random_dropentry(void);
 extern int ip_vs_conn_init(void);
 extern void ip_vs_conn_cleanup(void);
@@ -548,6 +564,7 @@ extern struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name);
 extern int sysctl_use_dst;
 extern int sysctl_ip_vs_cache_bypass;
 extern int sysctl_ip_vs_expire_nodest_conn;
+extern int sysctl_ip_vs_sync_threshold;
 extern atomic_t ip_vs_dropentry;
 extern struct ip_vs_stats ip_vs_stats;
 extern rwlock_t __ip_vs_svc_lock;
@@ -568,6 +585,16 @@ extern int ip_vs_control_init(void);
 extern void ip_vs_control_cleanup(void);
 
 
+/*
+ *      IPVS sync daemon data and function prototypes
+ *      (from ip_vs_sync.c)
+ */
+extern int ip_vs_sync_state;
+extern int start_sync_thread(struct ip_vs_rule_user *u);
+extern int stop_sync_thread(void);
+extern void ip_vs_sync_conn(struct ip_vs_conn *cp);
+
+
 /*
  *	This is a simple mechanism to ignore packets when
  *	we are loaded. Just set ip_vs_drop_rate to 'n' and
@@ -603,15 +630,21 @@ extern void ip_vs_sltimer_cleanup(void);
 
 extern __inline__ char ip_vs_fwd_tag(struct ip_vs_conn *cp)
 {
-	char fwd = 'M';
+	char fwd;
 
 	switch (IP_VS_FWD_METHOD(cp)) {
+	case IP_VS_CONN_F_MASQ:
+		fwd = 'M'; break;
 	case IP_VS_CONN_F_LOCALNODE:
 		fwd = 'L'; break;
 	case IP_VS_CONN_F_TUNNEL:
 		fwd = 'T'; break;
 	case IP_VS_CONN_F_DROUTE:
 		fwd = 'R'; break;
+	case IP_VS_CONN_F_BYPASS:
+		fwd = 'B'; break;
+	default:
+		fwd = '?'; break;
 	}
 	return fwd;
 }
@@ -657,6 +690,41 @@ __ip_vs_dst_check(struct ip_vs_dest *dest, u32 rtos, u32 cookie)
 	return dst;
 }
 
+static inline struct rtable *
+__ip_vs_get_out_rt(struct ip_vs_conn *cp, u32 rtos)
+{
+	struct rtable *rt;			/* Route to the other host */
+	struct ip_vs_dest *dest = cp->dest;
+
+	if (dest) {
+		spin_lock(&dest->dst_lock);
+		if (!sysctl_use_dst
+		    || !(rt = (struct rtable *)
+			 __ip_vs_dst_check(dest, rtos, 0))) {
+			if (ip_route_output(&rt, dest->addr, 0, rtos, 0)) {
+				spin_unlock(&dest->dst_lock);
+				IP_VS_DBG(0, "ip_route_output error, "
+					  "dest: %u.%u.%u.%u\n",
+					  NIPQUAD(dest->addr));
+				return NULL;
+			}
+			__ip_vs_dst_set(dest, rtos, dst_clone(&rt->u.dst));
+			IP_VS_DBG(10, "new dst %u.%u.%u.%u, refcnt=%d, rtos=%X\n",
+				  NIPQUAD(dest->addr),
+				  atomic_read(&rt->u.dst.__refcnt), rtos);
+		}
+		spin_unlock(&dest->dst_lock);
+	} else {
+		if (ip_route_output(&rt, cp->daddr, 0, rtos, 0)) {
+			IP_VS_DBG(0, "ip_route_output error, "
+				  "dest: %u.%u.%u.%u\n", NIPQUAD(dest->addr));
+			return NULL;
+		}
+	}
+
+	return rt;
+}
+
 
 #endif /* __KERNEL__ */
 
