@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.17 2001/06/04 13:51:03 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.18 2001/06/20 12:50:52 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -570,10 +570,13 @@ int ip_vs_set_state(struct ip_vs_conn *cp,
 	switch (iph->protocol) {
 	case IPPROTO_ICMP:
 		ret = vs_set_state_timeout(cp, IP_VS_S_ICMP);
+		break;
 	case IPPROTO_UDP:
 		ret = vs_set_state_timeout(cp, IP_VS_S_UDP);
+		break;
 	case IPPROTO_TCP:
 		ret = vs_tcp_state(cp, state_off, tp);
+		break;
 	default:
 		ret = -1;
 	}
@@ -608,7 +611,7 @@ static int ip_vs_bypass_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	EnterFunction(10);
 
 	if (ip_route_output(&rt, iph->daddr, 0, RT_TOS(tos), 0)) {
-		IP_VS_DBG(0, "ip_vs_bypass_xmit(): route error, "
+		IP_VS_DBG(0, "ip_vs_bypass_xmit(): ip_route_output error, "
 			  "dest: %u.%u.%u.%u\n", NIPQUAD(iph->daddr));
 		goto tx_error_icmp;
 	}
@@ -643,16 +646,6 @@ static int ip_vs_bypass_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 }
 
 
-/*
- *  Bind a connection entry with the passby_xmit.
- *  Called by ip_vs_leave.
- */
-void ip_vs_bind_bypass_xmit(struct ip_vs_conn *cp)
-{
-	cp->packet_xmit = ip_vs_bypass_xmit;
-}
-
-
 /*
  *      NULL transmitter (do nothing except return NF_ACCEPT)
  */
@@ -668,10 +661,7 @@ static int ip_vs_null_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 {
 	struct rtable *rt;			/* Route to the other host */
-	struct ip_vs_dest *dest;
 	struct iphdr  *iph = skb->nh.iph;
-	u8     tos = iph->tos;
-	u32    rtos;
 	union ip_vs_tphdr h;
 	unsigned short size;
 	int doff;                       /* transport protocol data offset */
@@ -713,7 +703,7 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	/*
 	 *  Check if it is no_cport connection (VS/NAT only)...
 	 */
-	if (cp->flags & IP_VS_CONN_F_NO_CPORT ) {
+	if (cp->flags & IP_VS_CONN_F_NO_CPORT) {
 		atomic_dec(&ip_vs_conn_no_cport_cnt);
 		ip_vs_conn_unhash(cp);
 		cp->flags &= ~IP_VS_CONN_F_NO_CPORT;
@@ -724,22 +714,8 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		IP_VS_DBG(10, "filled cport=%d\n", ntohs(cp->dport));
 	}
 
-	dest = cp->dest;
-	rtos = RT_TOS(tos);
-	spin_lock(&dest->dst_lock);
-	if (!sysctl_use_dst || !(rt = (struct rtable *) __ip_vs_dst_check(dest, rtos, 0))) {
-		if (ip_route_output(&rt, dest->addr, 0, rtos, 0)) {
-			spin_unlock(&dest->dst_lock);
-			IP_VS_DBG(0, "ip_vs_nat_xmit(): route error, "
-				  "dest: %u.%u.%u.%u\n", NIPQUAD(dest->addr));
-			goto tx_error_icmp;
-		}
-		__ip_vs_dst_set(dest, rtos, dst_clone(&rt->u.dst));
-		IP_VS_DBG(10, "NAT: new dst %u.%u.%u.%u, refcnt=%d, rtos=%X\n",
-			  NIPQUAD(dest->addr),
-			  atomic_read(&rt->u.dst.__refcnt), rtos);
-	}
-	spin_unlock(&dest->dst_lock);
+	if (!(rt = __ip_vs_get_out_rt(cp, RT_TOS(iph->tos))))
+		goto tx_error_icmp;
 
 	/* MTU checking */
 	mtu = rt->u.dst.pmtu;
@@ -852,11 +828,9 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 {
 	struct rtable *rt;			/* Route to the other host */
-	struct ip_vs_dest *dest;
 	struct net_device *tdev;		/* Device to other host */
 	struct iphdr  *old_iph = skb->nh.iph;
 	u8     tos = old_iph->tos;
-	u32    rtos;
 	u16    df = old_iph->frag_off;
 	struct iphdr  *iph;			/* Our new IP header */
 	int    max_headroom;			/* The extra header space needed */
@@ -871,22 +845,8 @@ static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		goto tx_error;
 	}
 
-	dest = cp->dest;
-	rtos = RT_TOS(tos);
-	spin_lock(&dest->dst_lock);
-	if (!sysctl_use_dst || !(rt = (struct rtable *) __ip_vs_dst_check(dest, rtos, 0))) {
-		if (ip_route_output(&rt, dest->addr, 0, rtos, 0)) {
-			spin_unlock(&dest->dst_lock);
-			IP_VS_DBG(0, "ip_vs_tunnel_xmit(): route error, "
-				  "dest: %u.%u.%u.%u\n", NIPQUAD(dest->addr));
-			goto tx_error_icmp;
-		}
-		__ip_vs_dst_set(dest, rtos, dst_clone(&rt->u.dst));
-		IP_VS_DBG(10, "TUN: new dst %u.%u.%u.%u, refcnt=%d, rtos=%X\n",
-			  NIPQUAD(dest->addr),
-			  atomic_read(&rt->u.dst.__refcnt), rtos);
-	}
-	spin_unlock(&dest->dst_lock);
+	if (!(rt = __ip_vs_get_out_rt(cp, RT_TOS(tos))))
+		goto tx_error_icmp;
 
 	tdev = rt->u.dst.dev;
 
@@ -978,30 +938,13 @@ static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 static int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 {
 	struct rtable *rt;			/* Route to the other host */
-	struct ip_vs_dest *dest;
 	struct iphdr  *iph = skb->nh.iph;
-	u8     tos = iph->tos;
-	u32    rtos;
 	int    mtu;
 
 	EnterFunction(10);
 
-	dest = cp->dest;
-	rtos = RT_TOS(tos);
-	spin_lock(&dest->dst_lock);
-	if (!sysctl_use_dst || !(rt = (struct rtable *) __ip_vs_dst_check(dest, rtos, 0))) {
-		if (ip_route_output(&rt, dest->addr, 0, rtos, 0)) {
-			spin_unlock(&dest->dst_lock);
-			IP_VS_DBG(0, "ip_vs_dr_xmit(): route error, "
-				  "dest: %u.%u.%u.%u\n", NIPQUAD(dest->addr));
-			goto tx_error_icmp;
-		}
-		__ip_vs_dst_set(dest, rtos, dst_clone(&rt->u.dst));
-		IP_VS_DBG(10, "DR: new dst %u.%u.%u.%u, refcnt=%d, rtos=%X\n",
-			  NIPQUAD(dest->addr),
-			  atomic_read(&rt->u.dst.__refcnt), rtos);
-	}
-	spin_unlock(&dest->dst_lock);
+	if (!(rt = __ip_vs_get_out_rt(cp, RT_TOS(iph->tos))))
+		goto tx_error_icmp;
 
 	/* MTU checking */
 	mtu = rt->u.dst.pmtu;
@@ -1038,24 +981,11 @@ static int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 
 
 /*
- *  Bind a connection entry with a virtual service destination
- *  Called just after a new connection entry is created.
+ *  Bind a connection entry with the corresponding packet_xmit.
+ *  Called by ip_vs_conn_new.
  */
-void ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
+static inline void ip_vs_bind_xmit(struct ip_vs_conn *cp)
 {
-	/* if dest is NULL, then return directly */
-	if (!dest)
-		return;
-
-	/*
-	 *    Increase the refcnt counter of the dest.
-	 */
-	atomic_inc(&dest->refcnt);
-
-	/* Bind with the destination and its corresponding transmitter */
-	cp->flags |= atomic_read(&dest->conn_flags);
-	cp->dest = dest;
-
 	switch (IP_VS_FWD_METHOD(cp)) {
 	case IP_VS_CONN_F_MASQ:
 		cp->packet_xmit = ip_vs_nat_xmit;
@@ -1072,7 +1002,33 @@ void ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
 	case IP_VS_CONN_F_LOCALNODE:
 		cp->packet_xmit = ip_vs_null_xmit;
 		break;
+
+	case IP_VS_CONN_F_BYPASS:
+		cp->packet_xmit = ip_vs_bypass_xmit;
+		break;
 	}
+}
+
+
+/*
+ *  Bind a connection entry with a virtual service destination
+ *  Called just after a new connection entry is created.
+ */
+static inline void
+ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
+{
+	/* if dest is NULL, then return directly */
+	if (!dest)
+		return;
+
+	/*
+	 *    Increase the refcnt counter of the dest.
+	 */
+	atomic_inc(&dest->refcnt);
+
+	/* Bind with the destination and its corresponding transmitter */
+	cp->flags |= atomic_read(&dest->conn_flags);
+	cp->dest = dest;
 
 	IP_VS_DBG(9, "Bind-dest %s c:%u.%u.%u.%u:%d v:%u.%u.%u.%u:%d "
 		  "d:%u.%u.%u.%u:%d fwd:%c s:%s flg:%X cnt:%d destcnt:%d\n",
@@ -1090,7 +1046,7 @@ void ip_vs_bind_dest(struct ip_vs_conn *cp, struct ip_vs_dest *dest)
  *  Unbind a connection entry with its VS destination
  *  Called by the ip_vs_conn_expire function.
  */
-void ip_vs_unbind_dest(struct ip_vs_conn *cp)
+static inline void ip_vs_unbind_dest(struct ip_vs_conn *cp)
 {
 	struct ip_vs_dest *dest = cp->dest;
 
@@ -1299,6 +1255,9 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
 	/* Set its state and timeout */
 	vs_set_state_timeout(cp, IP_VS_S_NONE);
 
+	/* Bind its packet transmitter */
+	ip_vs_bind_xmit(cp);
+
 	/*
 	 * Set the entry is referenced by the current thread before hashing
 	 * it in the table, so that other thread run ip_vs_random_dropentry
