@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.19 2001/06/04 13:51:03 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.20 2001/06/20 12:50:52 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -48,8 +48,6 @@
 
 EXPORT_SYMBOL(register_ip_vs_scheduler);
 EXPORT_SYMBOL(unregister_ip_vs_scheduler);
-EXPORT_SYMBOL(ip_vs_bind_dest);
-EXPORT_SYMBOL(ip_vs_unbind_dest);
 EXPORT_SYMBOL(ip_vs_skb_replace);
 EXPORT_SYMBOL(ip_vs_proto_name);
 EXPORT_SYMBOL(ip_vs_conn_new);
@@ -490,7 +488,8 @@ static int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
 		cp = ip_vs_conn_new(iph->protocol,
 				    iph->saddr, portp[0],
 				    iph->daddr, portp[1],
-				    0, 0, 0,
+				    0, 0,
+				    IP_VS_CONN_F_BYPASS,
 				    NULL);
 		if (cp == NULL) {
 			IP_VS_ERR("ip_vs_conn_new failed\n");
@@ -498,9 +497,6 @@ static int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
 			return NF_STOLEN;
 		}
 
-		/* bind the bypass_xmit */
-		ip_vs_bind_bypass_xmit(cp);
-
 		/* statistics */
 		ip_vs_in_stats(cp, skb);
 
@@ -510,6 +506,7 @@ static int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
 		/* transmit the first SYN packet */
 		ret = cp->packet_xmit(skb, cp);
 
+		atomic_inc(&cp->in_pkts);
 		ip_vs_conn_put(cp);
 		return ret;
 	}
@@ -900,9 +897,6 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 	unsigned short   len   = ntohs(iph->tot_len) - (iph->ihl * 4);
 	struct ip_vs_conn *cp;
 	struct rtable *rt;			/* Route to the other host */
-	struct ip_vs_dest *dest;
-	u8     tos = iph->tos;
-	u32    rtos;
 	int    mtu;
 
 	IP_VS_DBG(12, "icmp in (%d,%d) %u.%u.%u.%u -> %u.%u.%u.%u\n",
@@ -960,6 +954,7 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 			ret = cp->packet_xmit(skb, cp);
 		else
 			ret = NF_ACCEPT;
+		atomic_inc(&cp->in_pkts);
 		ip_vs_conn_put(cp);
 		return ret;
 	}
@@ -998,22 +993,8 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 	/*
 	 * Send the mangled packet immediately here
 	 */
-	dest = cp->dest;
-	rtos = RT_TOS(tos);
-	spin_lock(&dest->dst_lock);
-	if (!sysctl_use_dst || !(rt = (struct rtable *) __ip_vs_dst_check(dest, rtos, 0))) {
-		if (ip_route_output(&rt, dest->addr, 0, rtos, 0)) {
-			spin_unlock(&dest->dst_lock);
-			IP_VS_DBG(0, "ip_vs_in_imcp(): route error, "
-				  "dest: %u.%u.%u.%u\n", NIPQUAD(dest->addr));
-			goto tx_error_icmp;
-		}
-		__ip_vs_dst_set(dest, rtos, dst_clone(&rt->u.dst));
-		IP_VS_DBG(10, "ICMP: new dst %u.%u.%u.%u, refcnt=%d, rtos=%X\n",
-			  NIPQUAD(dest->addr), atomic_read(&rt->u.dst.__refcnt),
-			  rtos);
-	}
-	spin_unlock(&dest->dst_lock);
+	if (!(rt = __ip_vs_get_out_rt(cp, RT_TOS(iph->tos))))
+		goto tx_error_icmp;
 
 	/* MTU checking, is it necessary here??? */
 	mtu = rt->u.dst.pmtu;
@@ -1095,7 +1076,7 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 	}
 
 	if (iph->protocol == IPPROTO_ICMP)
-		return(ip_vs_in_icmp(skb_p));
+		return ip_vs_in_icmp(skb_p);
 
 	IP_VS_DBG(11, "Incoming %s %u.%u.%u.%u:%d->%u.%u.%u.%u:%d\n",
 		  ip_vs_proto_name(iph->protocol),
@@ -1167,6 +1148,17 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 			IP_VS_DBG(0, "warning: packet_xmit is null");
 			ret = NF_ACCEPT;
 		}
+
+		/* increase its packet counter and check if it is needed
+		   to be synchronized */
+		atomic_inc(&cp->in_pkts);
+		if (ip_vs_sync_state == IP_VS_STATE_MASTER
+		    && (cp->protocol != IPPROTO_TCP
+			|| cp->state == IP_VS_S_ESTABLISHED)
+		    && (atomic_read(&cp->in_pkts) % 50
+			== sysctl_ip_vs_sync_threshold))
+			ip_vs_sync_conn(cp);
+
 		ip_vs_conn_put(cp);
 		return ret;
 	}
