@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.17 2001/06/04 13:51:03 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.18 2001/06/20 12:50:52 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -76,6 +76,7 @@ static int sysctl_ip_vs_am_droprate = 10;
 int sysctl_use_dst = 1;
 int sysctl_ip_vs_cache_bypass = 0;
 int sysctl_ip_vs_expire_nodest_conn = 0;
+int sysctl_ip_vs_sync_threshold = 3;
 
 #ifdef CONFIG_IP_VS_DEBUG
 static int sysctl_ip_vs_debug_level = 0;
@@ -1290,7 +1291,7 @@ static int ip_vs_sysctl_defense_mode(ctl_table *ctl, int write,
  */
 struct ip_vs_sysctl_table {
 	struct ctl_table_header *sysctl_header;
-	ctl_table vs_vars[22];
+	ctl_table vs_vars[NET_IPV4_VS_LAST];
 	ctl_table vs_dir[2];
 	ctl_table ipv4_dir[2];
 	ctl_table root_dir[2];
@@ -1364,6 +1365,9 @@ static struct ip_vs_sysctl_table ipv4_vs_table = {
 	 {NET_IPV4_VS_EXPIRE_NODEST_CONN, "expire_nodest_conn",
 	  &sysctl_ip_vs_expire_nodest_conn, sizeof(int), 0644, NULL,
 	  &proc_dointvec},
+	 {NET_IPV4_VS_SYNC_THRESHOLD, "sync_threshold",
+	  &sysctl_ip_vs_sync_threshold, sizeof(int), 0644, NULL,
+	  &proc_dointvec},
 	 {0}},
 	{{NET_IPV4_VS, "vs", NULL, 0, 0555, ipv4_vs_table.vs_vars},
 	 {0}},
@@ -1768,8 +1772,15 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 		/* Set timeout values for (tcp tcpfin udp) */
 		ret = ip_vs_set_timeouts(urule);
 		goto out_unlock;
+	} else if (cmd == IP_VS_SO_SET_STARTDAEMON) {
+		ret = start_sync_thread(urule);
+		goto out_unlock;
+	} else if (cmd == IP_VS_SO_SET_STOPDAEMON) {
+		ret = stop_sync_thread();
+		goto out_unlock;
 	}
 
+
 	/*
 	 * Check for valid protocol: TCP or UDP. Even for fwmark!=0
 	 */
@@ -1838,10 +1849,10 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 	if (svc)
 		ip_vs_service_put(svc);
 
-  out_unlock:
+ out_unlock:
 	spin_unlock(&__ip_vs_mutex);
 	MOD_DEC_USE_COUNT;
-  out_free:
+ out_free:
 	kfree(urule);
 	return ret;
 }
