@@ -1,7 +1,7 @@
 /*
  * IPVS:        Least-Connection Scheduling module
  *
- * Version:     $Id: ip_vs_lc.c,v 1.6 2001/05/09 15:01:15 wensong Exp $
+ * Version:     $Id: ip_vs_lc.c,v 1.7 2001/06/04 13:51:03 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -57,10 +57,6 @@ ip_vs_lc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 
 	IP_VS_DBG(6, "ip_vs_lc_schedule(): Scheduling...\n");
 
-	l = &svc->destinations;
-	if (l == l->next)
-		return NULL;
-
 	/*
 	 * Simply select the server with the least number of
 	 *        (activeconns<<5) + inactconns
@@ -70,6 +66,7 @@ ip_vs_lc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 	 * served, but no new connection is assigned to the server.
 	 */
 
+	l = &svc->destinations;
 	for (e=l->next; e!=l; e=e->next) {
 		least = list_entry (e, struct ip_vs_dest, n_list);
 		if (atomic_read(&least->weight) > 0) {
